@@ -9,6 +9,7 @@ class Dataset:
     """ Parent class for common operations """
     id = None
     short_name = None
+    short_name_property = 'GEEDATASETS_SHORT_NAME'
     type = None
 
     def __init__(self, **kwargs):
@@ -35,7 +36,7 @@ Date: {date}
     bands = tuple()
     date = None
     region = None
-    visualizers = tuple()
+    visualizers = None
     extra_bands = tuple()
 
     def test(self, image=None, renamed=False, verbose=False):
@@ -165,7 +166,7 @@ Visualizers: {visualizers}
     start_date = None
     end_date = None
     region = None
-    visualizers = tuple()
+    visualizers = None
     masks = tuple()
     extra_bands = tuple()
 
@@ -237,10 +238,11 @@ Visualizers: {visualizers}
         params['masks'] = masks
         return self.INFO.format(**params)
 
-    def collection(self, bounds=None, date=None):
+    def collection(self, bounds=None, date=None, **kwargs):
         """ Google Earth Engine Original Image Collection.
         You can filter by bounds and date using parameters
         """
+        shortname_prop = kwargs.get('short_name_property', self.short_name_property)
         col = self.eeObject()
         if isinstance(bounds, (ee.Feature, ee.FeatureCollection, ee.Image)):
             bounds = bounds.geometry()
@@ -253,6 +255,10 @@ Visualizers: {visualizers}
             col = col.filterDate(ee.Date(date[0]), ee.Date(date[1]))
         elif isinstance(date, (str, ee.Date)):
             col = col.filterDate(ee.Date(date))
+
+        if shortname_prop:
+            col = col.map(lambda i: i.set(shortname_prop, self.short_name))
+
         return col
 
     def getMask(self, name):
@@ -405,7 +411,7 @@ Visualizers: {visualizers}
         original_names = {band.name: band.alias for band in self.bands}
         return geetools.tools.image.renameDict(image, original_names)
 
-    def proxyImage(self, renamed=False):
+    def proxyImage(self, renamed=False, masked=True):
         """ Create an Image with the band names, type and scale but empty """
         precisions = self.precisions(renamed=renamed)
         first_band = self.bands[0]
@@ -415,6 +421,8 @@ Visualizers: {visualizers}
             name = first_band.alias
 
         init = ee.Image.constant(0).rename(name)
+        if masked:
+            init = init.selfMask()
         init = convertPrecision(init, precisions[name])
         for i, band in enumerate(self.bands):
             if i == 0: continue
@@ -422,8 +430,9 @@ Visualizers: {visualizers}
                 name = band.name
             else:
                 name = band.alias
-
             img = ee.Image.constant(0).rename(name)
+            if masked:
+                img = img.selfMask()
             img = convertPrecision(img, precisions[name])
             init = init.addBands(img)
 
