@@ -3,7 +3,7 @@
 from .visualization import *
 from .datasets import OpticalSatellite, ImageCollection
 from .bands import OpticalBand, BitBand, ClassificationBand, ExpressionBand, \
-    RangeBand
+    RangeBand, Precisions
 from .masks import Mask
 from . import helpers
 import geetools
@@ -29,7 +29,7 @@ Visualizers: {visualizers}
     start_date = '2015-06-23'
     end_date = None
     cloud_cover = 'CLOUD_COVERAGE_ASSESSMENT'
-    _common = {'scale':0.0001, 'precision': 'uint16'}
+    _common = {'scale':0.0001, 'precision': Precisions.uint16}
     aerosol = OpticalBand('B1', 'aerosol', resolution=60, **_common)
     blue = OpticalBand('B2', 'blue', resolution=10, **_common)
     green = OpticalBand('B3', 'green', resolution=10, **_common)
@@ -43,9 +43,9 @@ Visualizers: {visualizers}
     swir = OpticalBand('B11', 'swir', resolution=20, **_common)
     swir2 = OpticalBand('B12', 'swir2', resolution=20, **_common)
     qa10 = BitBand(name='QA10', alias='qa10', bits=dict(), resolution=10,
-                   precision='uint16')
+                   precision=Precisions.uint16)
     qa20 = BitBand(name='QA20', alias='qa20', bits=dict(), resolution=20,
-                   precision='uint32')
+                   precision=Precisions.uint32)
     qa60 = BitBand(
         'QA60', 'qa60',
         precision='uint16',
@@ -63,11 +63,12 @@ Visualizers: {visualizers}
     ndre = ExpressionBand('NDRE', 'ndre', '(nir-red_edge_1)/(nir+red_edge_1)',
                           [nir, red_edge_1], precision='float')
 
-    visualizers = (
-        Visualization.NSR([nir, swir, red]),
-        Visualization.trueColor([red, green, blue]),
-        Visualization.falseColor([nir, red, green])
-    )
+    visualizers = Visualizers(
+        NSR = Visualization.NSR([nir, swir, red]),
+        TrueColor = Visualization.trueColor([red, green, blue]),
+        FalseColor = Visualization.falseColor([nir, red, green]),
+        RGB = Visualization.trueColor([red, green, blue]),
+      )
 
     extra_bands = (ndvi, nbr, ndre)
 
@@ -192,9 +193,9 @@ class Sentinel2SR(Sentinel2):
     short_name = 'S2SR'
     start_date = '2017-03-28'
     process = 'SR'
-    aot = OpticalBand('AOT', 'aerosol_thickness', precision='uint16',
+    aot = OpticalBand('AOT', 'aerosol_thickness', precision=Precisions.uint16,
                       resolution=10)
-    wvp = OpticalBand('WVP', 'water_vapor_pressure', precision='uint16',
+    wvp = OpticalBand('WVP', 'water_vapor_pressure', precision=Precisions.uint16,
                       resolution=10)
 
     scl = ClassificationBand(
@@ -221,24 +222,24 @@ class Sentinel2SR(Sentinel2):
                    'cirrus', 'snow']
     )
     tci_r = OpticalBand(name='TCI_R', alias='true_color_red',
-                        precision='uint8', resolution=10)
+                        precision=Precisions.uint8, resolution=10)
     tci_g = OpticalBand(name='TCI_G', alias='true_color_blue',
-                        precision='uint8', resolution=10)
+                        precision=Precisions.uint8, resolution=10)
     tci_b = OpticalBand(name='TCI_B', alias='true_color_green',
-                        precision='uint8', resolution=10)
+                        precision=Precisions.uint8, resolution=10)
 
     cloud_prob = OpticalBand('MSK_CLDPRB', 'cloud_probability', 'percentage',
-                             precision='uint8', resolution=20)
+                             precision=Precisions.uint8, resolution=20)
     snow_prob = OpticalBand('MSK_SNWPRB', 'snow_probability', 'percentage',
-                             precision='uint8', resolution=10)
+                             precision=Precisions.uint8, resolution=10)
 
 
-    # Add visualizer
-    visualizers = (
-        Visualization.NSR([Sentinel2.nir, Sentinel2.swir, Sentinel2.red]),
-        Visualization.trueColor([Sentinel2.red, Sentinel2.green, Sentinel2.blue]),
-        Visualization.falseColor([Sentinel2.nir, Sentinel2.red, Sentinel2.green]),
-        Visualization('SCL', [scl], 1, 11, palette=[
+    # Add visualizers
+    visualizers = Visualizers(
+        NSR = Visualization.NSR([Sentinel2.nir, Sentinel2.swir, Sentinel2.red]),
+        TrueColor = Visualization.trueColor([Sentinel2.red, Sentinel2.green, Sentinel2.blue]),
+        FalseColor = Visualization.falseColor([Sentinel2.nir, Sentinel2.red, Sentinel2.green]),
+        SCL = Visualization('SCL', [scl], 1, 11, palette=[
             '#ff0004', '#868686', '#774b0a', '#10d22c', '#ffff52', '#0000ff',
             '#818181', '#c0c0c0', '#f1f1f1', '#bac5eb', '#52fff9'
         ])
@@ -272,11 +273,12 @@ class Sentinel1(ImageCollection):
     start_date = '2014-10-03'
     end_date = None
     polarizations = ('HH', 'HH-HV', 'VV', 'VV-VH')
-    HH = OpticalBand('HH', 'HH', 'db', precision='double', resolution=10)
-    HV = OpticalBand('HV', 'HV', 'db', precision='double', resolution=10)
-    VH = OpticalBand('VH', 'VH', 'db', precision='double', resolution=10)
-    VV = OpticalBand('VV', 'VV', 'db', precision='double', resolution=10)
-    angle = OpticalBand('angle', 'angle', 'degrees', precision='float', resolution=10)
+    HH = OpticalBand('HH', 'HH', 'db', precision=Precisions.double, resolution=10)
+    HV = OpticalBand('HV', 'HV', 'db', precision=Precisions.double, resolution=10)
+    VH = OpticalBand('VH', 'VH', 'db', precision=Precisions.double, resolution=10)
+    VV = OpticalBand('VV', 'VV', 'db', precision=Precisions.double, resolution=10)
+    angle = OpticalBand('angle', 'angle', 'degrees',
+                        precision=Precisions.float, resolution=10)
     bands = (HH, HV, VH, VV, angle)
     masks = (Mask.empty(), )
 
@@ -288,7 +290,7 @@ class Sentinel1(ImageCollection):
         ( (10**(VV/10)) + (10**(VH/10)) )
         """,
         [VV, VH],
-        precision='float',
+        precision=Precisions.float,
         resolution=10
     )
 
@@ -518,8 +520,10 @@ class Sentinel1(ImageCollection):
             f = self._lee_filter(im.select(band))
             return im.addBands(f, overwrite=True)
 
-        filtered = ee.Image(bands.iterate(wrap, image))
-        db = self.convert_to_db(filtered)
+        filtered = ee.Image(bands.iterate(wrap, image))\
+                     .copyProperties(source=image)\
+                     .set('system:time_start', image.date().millis())
+        db = self.convert_to_db(ee.Image(filtered))
         return db
 
 
@@ -530,14 +534,14 @@ class S2Cloudless(ImageCollection):
     start_date = '2015-06-23'
     end_date = None
     probability = RangeBand('probability', 'probability', 0, 100,
-                            'uint8', 10, 'percentage', 1)
+                            Precisions.uint8, 10, 'percentage', 1)
     bands = (probability,)
-    band = 'cloud_probability'
 
-    def match(self, collection):
+    def addProbability(self, collection, rename=False):
         """ Combine this collection with the given S2 collection """
         # Join S2 SR with cloud probability dataset to add cloud mask.
-        matched = ee.Join.saveFirst(self.band).apply(
+        name = self.probability.alias if rename else self.probability.name
+        matched = ee.Join.saveFirst(name).apply(
             primary=collection,
             secondary=self.collection(),
             condition=ee.Filter.equals(leftField='system:index',
@@ -545,18 +549,25 @@ class S2Cloudless(ImageCollection):
         )
 
         def wrap(image):
-            match = ee.Image(image.get(self.band))
-            match = match.rename(self.band)
+            match = ee.Image(image.get(name))
+            match = match.rename(name)
             return ee.Image(image).addBands(match)
 
         return ee.ImageCollection(matched.map(wrap))
 
-    def apply(self, collection, threshold):
+    def apply(self, image, threshold, renamed=False):
+        """ Apply the mask to an image using the
+            given threshold. If the probability is higher than the threshold, it's
+            considered as cloud and therefore masked """
+        image = ee.Image(image)
+        name = self.probability.alias if renamed else self.probability.name
+        mask = image.select(name).lte(threshold)
+        return ee.Image(image.updateMask(mask))
+
+    def applyCollection(self, collection, threshold, renamed=False):
         """ Apply the mask to the images in a collection using the
-        given threshold """
-        matched = self.match(collection)
-        def wrap(image):
-            image = ee.Image(image)
-            mask = image.select(self.band).lte(threshold)
-            return ee.Image(image.updateMask(mask))
-        return ee.ImageCollection(matched.map(wrap))
+        given threshold. If the probability is higher than the threshold, it's
+        considered as cloud and therefore masked """
+        # name = self.probability.alias if renamed else self.probability.name
+        matched = self.addProbability(collection, renamed)
+        return ee.ImageCollection(matched.map(lambda i: self.apply(i, threshold, renamed)))
